@@ -18,13 +18,13 @@
 extern "C" {
 #endif
 
-void proxyDebugInit(AGClientProcessor *cp);
-int  proxyDebug(AGClientProcessor *cp, const char *fmt, ...);
-void proxyDebugCleanup(AGClientProcessor *cp);
+ExportFunc void proxyDebugInit(AGClientProcessor *cp);
+ExportFunc int  proxyDebug(AGClientProcessor *cp, const char *fmt, ...);
+ExportFunc void proxyDebugCleanup(AGClientProcessor *cp);
 
 #ifdef __cplusplus
 }
 #endif 
 
 #endif
-#endif 
\ No newline at end of file
+#endif
