@@ -1,4 +1,4 @@
-/* $Id: mal-conduit.c,v 1.7 2001/08/02 08:47:44 eskil Exp $ */
+/* $Id: mal-conduit.c,v 1.11 2005/02/03 22:04:20 jpr Exp $ */
 
 #include <glib.h>
 #include <gnome.h>
@@ -34,43 +34,46 @@
 
 #include "mal-conduit.h"
 
+#define MAL_CONDUIT_VERSION "0.9-2.0.4"
+
 #define OBJ_DATA_CONFIG  "conduit_config"
 #define OBJ_DATA_OLDCONFIG  "conduit_oldconfig"
 #define OBJ_DATA_CONFIG_WINDOW  "config_window"
 
-#define GET_CONDUIT_CFG(s) ((ConduitCfg*)gtk_object_get_data(GTK_OBJECT(s),OBJ_DATA_CONFIG))
-#define GET_CONDUIT_OLDCFG(s) ((ConduitCfg*)gtk_object_get_data(GTK_OBJECT(s),OBJ_DATA_OLDCONFIG))
-#define GET_CONDUIT_WINDOW(s) ((GtkWidget*)gtk_object_get_data(GTK_OBJECT(s),OBJ_DATA_CONFIG_WINDOW))
+#define GET_CONDUIT_CFG(s) ((ConduitCfg*)gtk_object_get_data (GTK_OBJECT (s),OBJ_DATA_CONFIG))
+#define GET_CONDUIT_OLDCFG(s) ((ConduitCfg*)gtk_object_get_data (GTK_OBJECT (s),OBJ_DATA_OLDCONFIG))
+#define GET_CONDUIT_WINDOW(s) ((GtkWidget*)gtk_object_get_data (GTK_OBJECT (s),OBJ_DATA_CONFIG_WINDOW))
 
 #define CONFIG_PREFIX "/gnome-pilot.d/mal-conduit/Pilot_%u/"
 
 /* This structure is taken from MAL malsync.c. It is used to interact
    with various MAL calls */
 typedef struct {
-    AGDeviceInfo * deviceInfo;
-    AGUserConfig * userConfig;
-    AGServerConfig * currentServerConfig;
-    AGClientProcessor * clientProcessor;
-
-    AGPlatformCalls * platform;
-    AGRecord * record;
-    AGDBConfig * currentDb;
-    AGCommandProcessor *commandProcessor;
-    AGBool quit;
-    
-    /* Pilot-specific */
-    AGBool currentDBIsResourceType;
-    int pilot_rHandle;
-    int pilot_RecIndex;
-    recordid_t id;
-    uint8  *pilot_buffer;
-    int pilot_buffer_size;
-    
-    /* Secure Network Library Stuff */
-    AGBool          hasseclib;
-
+	AGDeviceInfo * deviceInfo;
+	AGUserConfig * userConfig;
+	AGServerConfig * currentServerConfig;
+	AGClientProcessor * clientProcessor;
+	
+	AGPlatformCalls * platform;
+	AGRecord * record;
+	AGDBConfig * currentDb;
+	AGCommandProcessor *commandProcessor;
+	AGBool quit;
+	
+	/* Pilot-specific */
+	AGBool currentDBIsResourceType;
+	int pilot_rHandle;
+	int pilot_RecIndex;
+	recordid_t id;
+	uint8  *pilot_buffer;
+	int pilot_buffer_size;
+	
+	/* Secure Network Library Stuff */
+	AGBool          hasseclib;
+	
 	/* We're smuggling the GnomePilotConduit along for the ride */
-	GnomePilotConduit *conduit;
+	GnomePilotConduitStandard *conduit;
+
 } PalmSyncInfo;
 
 typedef struct  {
@@ -79,6 +82,11 @@ typedef struct  {
 
 	gboolean ok_to_sync;
 	guint32 pilot_id;
+
+	gchar *proxy_address, *proxy_username, *proxy_password;
+	gint proxy_port;
+	gchar *socks_proxy_address;
+	gint socks_proxy_port;
 } ConduitCfg;
 
 typedef sword (*netInitFunc)(AGNetCtx *ctx);
@@ -95,15 +103,29 @@ typedef void  (*netPostSyncHook) (AGNetCtx *ctx,
                                  AGSyncProcessor *sp,
                                  AGBool connectSecure);
 
+/* prototype to get rid of warnings */
+extern PalmSyncInfo* syncInfoNew ();
+extern AGUserConfig* getUserConfig(uint32*);
+
 extern netInitFunc     secnetinit;
 extern netCloseFunc    secnetclose;
 extern netCtxSizeFunc  secctxsize;
 extern netPreSyncHook  secnetpresync;
 extern netPostSyncHook  secnetpostsync;
 
+extern char *httpProxy;
+extern int   httpProxyPort;
+extern char *socksProxy;
+extern int   socksProxyPort;
+extern char *proxyUsername;
+extern char *proxyPassword;
+
 extern int sd;
 static int daemon_mode = 0;
-static int verbose = 1;
+
+/* I need this as a global, since I can't pass data
+   to malsync callbacks */
+GnomePilotConduit *the_conduit;
 
 static void
 bonk_sync_date (ConduitCfg *cfg)
@@ -118,21 +140,49 @@ bonk_sync_date (ConduitCfg *cfg)
 	g_free (prefix);
 }
 
-static gint 
-synchronize( GnomePilotConduit *c, 
-	     GnomePilotDBInfo *dbi ) 
+#if 1
+/* This is used in mal/client/unix/malsync.c:doStartServer(...) */
+int32
+cmdTASK (void *out, int32 err_code, char *task, AGBool bufferable) {
+	if (task) {
+		g_message ("TASK : %s", task);
+	}
+	gnome_pilot_conduit_send_message (the_conduit, task);
+	return AGCLIENT_CONTINUE;
+}
+
+/* This is used in mal/client/unix/malsync.c:doStartServer(...) */
+int32 
+cmdITEM (void *out, int32 *returnErrorCode, int32 currentItemNumber,
+	 int32 totalItemCount, char *currentItem)
+{
+	g_message ("%d/%d (%s)", currentItemNumber, totalItemCount, currentItem);
+	if (currentItemNumber == totalItemCount)
+		g_message ("done");
+	
+	if (totalItemCount > 0) {
+		gnome_pilot_conduit_send_progress (the_conduit, totalItemCount, currentItemNumber);
+	}
+
+	return AGCLIENT_CONTINUE;
+}
+#endif 
+
+static int 
+synchronize (GnomePilotConduitStandard *c, 
+	     GnomePilotDBInfo *dbi,
+	     gpointer unused) 
 {
 	PalmSyncInfo *pInfo;
 	uint32 pilotID;
 	AGNetCtx *ctx;
 	ConduitCfg *cfg;
 
-	g_message("MALconduit 0.7");
+	g_message ("MALconduit %s", MAL_CONDUIT_VERSION);
 	
-
 	cfg = GET_CONDUIT_CFG (c);
 	if (cfg->once_a_day && !cfg->ok_to_sync) {
-		gnome_pilot_conduit_send_message(GNOME_PILOT_CONDUIT(c), 
+		gnome_pilot_conduit_send_message (GNOME_PILOT_CONDUIT (c), 
 						 _("Already synchronized today"));
 		g_message ("already synchronized today");
 		return -1;
@@ -143,29 +193,41 @@ synchronize( GnomePilotConduit *c,
 	pilotID = cfg->pilot_id;
 	sd = dbi->pilot_socket;
 
-	pInfo = syncInfoNew();
+	pInfo = syncInfoNew ();
 	pInfo->conduit = c;
-	if (!loadSecLib(&ctx)) {
-	  ctx = (AGNetCtx *)malloc(sizeof(AGNetCtx));
-	  AGNetInit(ctx);
+	if (!loadSecLib (&ctx)) {
+	  ctx = (AGNetCtx *)malloc (sizeof (AGNetCtx));
+	  AGNetInit (ctx);
 	} 
 
-	if(setupPlatformCalls(pInfo)) {
+	if (setupPlatformCalls (pInfo)) {
 		return -1;
 	}
 
-	pInfo->userConfig = getUserConfig(&pilotID);
-	doClientProcessorLoop(pInfo, ctx);
-	storeDeviceUserConfig(pInfo->userConfig, pilotID);
+	if (cfg->proxy_address || cfg->socks_proxy_address) {
+		g_message ("trying to set proxy stuff...");
+		httpProxy = cfg->proxy_address;
+		httpProxyPort = cfg->proxy_port;
+		socksProxy = cfg->socks_proxy_address;
+		socksProxyPort = cfg->socks_proxy_port;
+		proxyUsername = cfg->proxy_username;
+		proxyPassword = cfg->proxy_password;
+	}
+
+	pInfo->userConfig = getUserConfig (&pilotID);
+	if (doClientProcessorLoop (pInfo, ctx) == TRUE) {
+		bonk_sync_date (cfg);
+	}
+	storeDeviceUserConfig (pInfo->userConfig, pilotID);
 
         if (secnetclose) {
             (*secnetclose)(ctx);
         } else {
-            AGNetClose(ctx);
+            AGNetClose (ctx);
         }
         
-        syncInfoFree(pInfo);
-        free(ctx);
+        syncInfoFree (pInfo);
+        free (ctx);
 	
 	return 0;
 }
@@ -184,6 +246,12 @@ load_config (ConduitCfg **cfg, GPilotPilot *pilot)
 	gnome_config_push_prefix (prefix);
 	(*cfg)->once_a_day = gnome_config_get_bool ("once_a_day=TRUE");
 	(*cfg)->last_sync_date = (time_t)gnome_config_get_int ("last_sync=1");
+	(*cfg)->proxy_address = gnome_config_get_string ("proxy_address");
+	(*cfg)->proxy_username = gnome_config_get_string ("proxy_username");
+	(*cfg)->proxy_password = gnome_config_get_string ("proxy_password");
+	(*cfg)->proxy_port = gnome_config_get_int ("proxy_port");
+	(*cfg)->socks_proxy_address = gnome_config_get_string ("socks_proxy_address");
+	(*cfg)->socks_proxy_port = gnome_config_get_int ("socks_proxy_port");
 	last_sync = gmtime (&((*cfg)->last_sync_date));	
 	(*cfg)->pilot_id = pilot->pilot_id;
 
@@ -191,6 +259,10 @@ load_config (ConduitCfg **cfg, GPilotPilot *pilot)
 	time (&t);
 	today = gmtime (&t);
 
+	g_message ("Last sync was day %d, today is %d", 
+		   last_day,
+		   today->tm_yday);
+
 	if (today->tm_yday != last_day) {
 		(*cfg)->ok_to_sync = TRUE;
 	} else {
@@ -211,6 +283,17 @@ save_config (ConduitCfg *cfg)
 	gnome_config_set_bool ("once_a_day", cfg->once_a_day);
 	gnome_config_set_int ("last_sync", cfg->last_sync_date);
 
+	if (cfg->proxy_address) {
+		gnome_config_set_string ("proxy_address", cfg->proxy_address);		
+		gnome_config_set_string ("proxy_username", cfg->proxy_username);
+		gnome_config_set_string ("proxy_password", cfg->proxy_password);
+		gnome_config_set_int ("proxy_port", cfg->proxy_port);
+	}
+	if (cfg->socks_proxy_address) {
+		gnome_config_set_string ("socks_proxy_address", cfg->socks_proxy_address);
+		gnome_config_set_int ("socks_proxy_port", cfg->socks_proxy_port);
+	}
+
 	gnome_config_pop_prefix ();
 	g_free (prefix);
 }
@@ -239,52 +322,52 @@ copy_config (ConduitCfg *to, ConduitCfg *from)
 }
 
 static GtkWidget
-*createCfgWindow(GnomePilotConduit* conduit)
+*createCfgWindow (GnomePilotConduit* conduit)
 {
 	GtkWidget *vbox, *table;
 	GtkWidget *label;
 	GtkWidget *button;	
 
-	vbox = gtk_vbox_new(FALSE, GNOME_PAD);
+	vbox = gtk_vbox_new (FALSE, GNOME_PAD);
 
-	table = gtk_table_new(2, 1, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
+	table = gtk_table_new (2, 1, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 10);
+	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, GNOME_PAD);
 
-	label = gtk_label_new(_("Only sync MAL once a day"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
+	label = gtk_label_new (_("Only sync MAL once a day"));
+	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-	button = gtk_check_button_new();
-	gtk_object_set_data(GTK_OBJECT(vbox), "only_once_a_day", button);
-	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
+	button = gtk_check_button_new ();
+	gtk_object_set_data (GTK_OBJECT (vbox), "only_once_a_day", button);
+	gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
 
 	return vbox;
 }
 
 static void
-setOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
+setOptionsCfg (GtkWidget *pilotcfg, ConduitCfg *state)
 {
 	GtkWidget *once_a_day;
 	GtkObject *adj;
 
 	once_a_day = gtk_object_get_data (GTK_OBJECT (pilotcfg), "only_once_a_day");
 
-	g_assert(once_a_day != NULL);
+	g_assert (once_a_day != NULL);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(once_a_day), state->once_a_day);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (once_a_day), state->once_a_day);
 }
 
 
 static void
-readOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
+readOptionsCfg (GtkWidget *pilotcfg, ConduitCfg *state)
 {
 	GtkWidget *once_a_day;
 	GtkObject *adj;
 
 	once_a_day = gtk_object_get_data (GTK_OBJECT (pilotcfg), "only_once_a_day");
 
-	g_assert(once_a_day != NULL);
+	g_assert (once_a_day != NULL);
 
 	state->once_a_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (once_a_day));
 }
@@ -322,14 +405,11 @@ static void
 revert_settings  (GnomePilotConduit *conduit, gpointer data)
 {
 	ConduitCfg *cfg,*cfg2;
-	cfg2= GET_CONDUIT_OLDCFG(conduit);
-	cfg = GET_CONDUIT_CFG(conduit);
+	cfg2= GET_CONDUIT_OLDCFG (conduit);
+	cfg = GET_CONDUIT_CFG (conduit);
 	save_config (cfg2);
 	copy_config (cfg,cfg2);
-	setOptionsCfg(GET_CONDUIT_WINDOW(conduit),cfg);
-}
-
-static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi ) {
+	setOptionsCfg (GET_CONDUIT_WINDOW (conduit),cfg);
 }
 
 GnomePilotConduit *conduit_load_gpilot_conduit (GPilotPilot *pilot) 
@@ -337,43 +417,36 @@ GnomePilotConduit *conduit_load_gpilot_conduit (GPilotPilot *pilot)
   GtkObject *retval;
   ConduitCfg *cfg, *cfg2;
 
-  retval = gnome_pilot_conduit_standard_new("AvantGo",0x4176476f, NULL);
-  g_assert(retval != NULL);
-
+  retval = gnome_pilot_conduit_standard_new ("AvantGo",0x4176476f, NULL);
+  g_assert (retval != NULL);
+  the_conduit = GNOME_PILOT_CONDUIT (retval);
 
   load_config (&cfg, pilot);
   cfg2 = dupe_config (cfg);
   
-  gtk_object_set_data (GTK_OBJECT(retval), OBJ_DATA_CONFIG, cfg);
-  gtk_object_set_data (GTK_OBJECT(retval), OBJ_DATA_OLDCONFIG, cfg2);
+  gtk_object_set_data (GTK_OBJECT (retval), OBJ_DATA_CONFIG, cfg);
+  gtk_object_set_data (GTK_OBJECT (retval), OBJ_DATA_OLDCONFIG, cfg2);
+
   
-  gtk_signal_connect(retval, 
-		     GNOME_PILOT_CONDUIT_STANDARD_SIGNAL_SYNCHRONIZE (synchronize),
-		     NULL);
-  gtk_signal_connect (retval, 
-		      GNOME_PILOT_CONDUIT_SIGNAL_CREATE_SETTINGS_WINDOW (create_settings_window),
-		      NULL);
-  gtk_signal_connect (retval, 
-		      GNOME_PILOT_CONDUIT_SIGNAL_DISPLAY_SETTINGS (display_settings),
-		      NULL);
-  gtk_signal_connect (retval,
-		      GNOME_PILOT_CONDUIT_SIGNAL_SAVE_SETTINGS (save_settings),
-		      NULL);
-  gtk_signal_connect (retval, 
-		      GNOME_PILOT_CONDUIT_SIGNAL_REVERT_SETTINGS (revert_settings),
-		      NULL);
-  return GNOME_PILOT_CONDUIT(retval); 
+  gtk_signal_connect (retval, "synchronize", (GtkSignalFunc)synchronize, NULL);
+
+  gtk_signal_connect (retval, "create_settings_window", (GtkSignalFunc) create_settings_window, NULL);
+  gtk_signal_connect (retval, "display_settings", (GtkSignalFunc) display_settings, NULL);
+  gtk_signal_connect (retval, "save_settings", (GtkSignalFunc) save_settings, NULL);
+  gtk_signal_connect (retval, "revert_settings", (GtkSignalFunc) revert_settings, NULL);
+
+  return GNOME_PILOT_CONDUIT (retval); 
 }
 
 
-void conduit_destroy_gpilot_conduit( GnomePilotConduit *c ) 
+void conduit_destroy_gpilot_conduit ( GnomePilotConduit *c ) 
 {
 #if 0
   ConduitCfg *cc;
   
-  cc = GET_CONFIG(c);
-  destroy_configuration( &cc );
+  cc = GET_CONFIG (c);
+  destroy_configuration ( &cc );
 #endif
-  gtk_object_destroy(GTK_OBJECT(c));
+  gtk_object_destroy (GTK_OBJECT (c));
 }
 
